package com.sun.xml.internal.bind.v2.runtime.property;

import java.util.Collection;
import com.sun.xml.internal.bind.v2.model.core.ID;
import com.sun.xml.internal.bind.v2.model.runtime.RuntimeNonElement;
import com.sun.xml.internal.bind.v2.model.runtime.RuntimeTypeInfo;
import com.sun.xml.internal.bind.v2.model.core.PropertyKind;
import java.lang.reflect.InvocationTargetException;
import com.sun.xml.internal.bind.v2.model.runtime.RuntimeElementPropertyInfo;
import com.sun.xml.internal.bind.v2.model.runtime.RuntimeValuePropertyInfo;
import com.sun.xml.internal.bind.v2.model.runtime.RuntimeAttributePropertyInfo;
import com.sun.xml.internal.bind.v2.model.runtime.RuntimePropertyInfo;
import com.sun.xml.internal.bind.v2.runtime.JAXBContextImpl;
import java.lang.reflect.Constructor;

public abstract class PropertyFactory
{
    private static final Constructor<? extends Property>[] propImpls;
    
    private PropertyFactory() {
    }
    
    public static Property create(JAXBContextImpl grammar, RuntimePropertyInfo info) {
        PropertyKind kind = info.kind();
        switch (kind) {
            case ATTRIBUTE: {
                return new AttributeProperty(grammar, (RuntimeAttributePropertyInfo)info);
            }
            case VALUE: {
                return new ValueProperty(grammar, (RuntimeValuePropertyInfo)info);
            }
            case ELEMENT: {
                if (((RuntimeElementPropertyInfo)info).isValueList()) {
                    return new ListElementProperty(grammar, (RuntimeElementPropertyInfo)info);
                }
            }
            case REFERENCE:
            case MAP: {
                break;
            }
            default: {
                assert false;
                break;
            }
        }
        boolean isCollection = info.isCollection();
        boolean isLeaf = isLeaf(info);
        Constructor<? extends Property> c = PropertyFactory.propImpls[(isLeaf ? 0 : 6) + (isCollection ? 3 : 0) + kind.propertyIndex];
        try {
            return (Property)c.newInstance(new Object[] { grammar, info });
        }
        catch (InstantiationException e) {
            throw new InstantiationError(e.getMessage());
        }
        catch (IllegalAccessException e2) {
            throw new IllegalAccessError(e2.getMessage());
        }
        catch (InvocationTargetException e3) {
            Throwable t = e3.getCause();
            if ((t instanceof Error)) {
                throw (Error)t;
            }
            if ((t instanceof RuntimeException)) {
                throw (RuntimeException)t;
            }
            throw new AssertionError(t);
        }
    }
    
    static boolean isLeaf(RuntimePropertyInfo info) {
        Collection<? extends RuntimeTypeInfo> types = info.ref();
        if (types.size() != 1) {
            return false;
        }
        RuntimeTypeInfo rti = (RuntimeTypeInfo)types.iterator().next();
        return rti instanceof RuntimeNonElement && (info.id() == ID.IDREF || (((RuntimeNonElement)rti).getTransducer() != null && info.getIndividualType().equals(rti.getType())));
    }
    
    static {
        Class<? extends Property>[] implClasses = { SingleElementLeafProperty.class, null, null, ArrayElementLeafProperty.class, null, null, SingleElementNodeProperty.class, SingleReferenceNodeProperty.class, SingleMapNodeProperty.class, ArrayElementNodeProperty.class, ArrayReferenceNodeProperty.class, null };
        propImpls = new Constructor[implClasses.length];
        for (int i = 0; i < PropertyFactory.propImpls.length; ++i) {
            if (implClasses[i] != null) {
                PropertyFactory.propImpls[i] = implClasses[i].getConstructors()[0];
            }
        }
    }
}
